# -*- coding: UTF-8 -*-
import numpy as np
from math import *

############################
# 関数に対するプログラム例 #
#      coded by Y.Suganuma #
############################

##################################
# 2 つの整数の和                 #
#      u, v : 和を計算するデータ #
#      t : 文字列                #
#      return : u + v            #
##################################
def func1(u, v, t) :
	t = t.replace("a", "x")
	u = 100
	a = u + v
	return a

########################
# 1 次元配列の要素の和 #
#      n : 要素数      #
#      u, v : 配列     #
########################
def func2(n, u, v) :
	for i1 in range(0, n) :
		u[n] += u[i1]
		v[n] += v[i1]

########################
# 2 次元配列の要素の和 #
#      n : 行数        #
#      m : 列数        #
#      u : 配列        #
########################
def func3(n, m, u) :
	for i1 in range(0, n) :
		for i2 in range(0, m) :
			u[i1][m] += u[i1][i2];

##################
# クラス Example #
##################
class Example :
	def init(self, n) :
		self.n = n
		self.x = [1, 2, 3]

##########################
# オブジェクトの値変更   #
#      x1 : オブジェクト #
##########################
def func4(x1) :
	x1.n    = 4
	x1.x[0] = 5

########################
# データの比較（昇順） #
# a > bの場合に真      #
########################
def ascend(a, b) :
	tf = False
	if a > b :
		tf = True
	return tf

########################
# データの比較（降順） #
# a < bの場合に真      #
########################
def descend(a, b) :
	tf = False
	if a < b :
		tf = True
	return tf

#################################################################
# バブルソート                                                  #
#      n : データの数                                           #
#      data : データ                                            #
#      compare : 比較を行う関数                                 #
# バブルソートの手順                                            #
#　　　　　1)配列data[i]にn個のデータが入っているものとする．   #
#　　　　　　　　i=0,1,･･･,n-1                                  #
#　　　　　2)i番目のデータとi+1番目のデータを比較し，もし，     #
#　　　　　　　　data[i+1] < data[i]　（昇順の場合）            #
#　　　　　　であればそれらのデータを交換する．　i=0,1,･･･,n-2  #
#　　　　　　　　→この結果，data[n-1]に最も大きなデータが入る  #
#              ことになる                                       #
#　　　　　3)次に，n-1個のデータに対し上の処理を繰り返す．      #
#　　　　　4)同様に，n-2，n-3，・・・個のデータに対して同じ処理 #
#        を繰り返す                                             #
#################################################################
def func5(n, data, compare) :
	sw = 0;

	if n > 1 :
		for i1 in range(0, n-1) :
			if compare(data[i1], data[i1+1]) : 
				sw         = 1
				x          = data[i1]
				data[i1]   = data[i1+1]
				data[i1+1] = x

	if sw > 0 :
		func5(n-1, data, compare)

########################
# 関数値（f(x)）の計算 #
########################
def snx(x) :
	return exp(x) - 3.0 * x

####################
# 関数の微分の計算 #
####################
def dsnx(x) :
	return exp(x) - 3.0

#####################################################
# Newton法による非線形方程式(f(x)=0)の解            #
#      fn : f(x)を計算する関数名                    #
#      dfn : f(x)の微分を計算する関数名             #
#      x0 : 初期値                                  #
#      eps1 : 終了条件１（｜x(k+1)-x(k)｜＜eps1）   #
#      eps2 : 終了条件２（｜f(x(k))｜＜eps2）       #
#      max : 最大試行回数                           #
#      ind : 実際の試行回数                         #
#            （負の時は解を得ることができなかった） #
#      return : 解                                  #
#####################################################
def func6(f, df, x0, eps1, eps2, max, ind) :
	x      = x0
	x1     = x0
	sw     = 0
	ind[0] = 0

	while sw == 0 and ind[0] >= 0 :
		sw      = 1
		ind[0] += 1
		g       = f(x1)
		if abs(g) > eps2 :
			if ind[0] <= max :
				dg = df(x1);
				if abs(dg) > eps2 :
					x = x1 - g / dg
					if abs(x-x1) > eps1 and abs(x-x1) > eps1*abs(x) :
						x1 = x
						sw = 0
				else :
					ind[0] = -1
			else :
				ind[0] = -1

	return x

			#
			# 基本型，文字列
			#
print("基本型，文字列")
a1 = 10
b1 = 20
c1 = 0
s1 = "abc"
print("   前", a1, b1, c1, s1)
c1 = func1(a1, b1, s1)
print("   後", a1, b1, c1, s1)
			#
			# 1 次元配列
			#
print("1 次元配列")
a2 = [1, 2, 3, 0]
b2 = np.array([1, 2, 3, 0])
print("   前 a2", a2[0], a2[1], a2[2], a2[3])
print("      b2", b2[0], b2[1], b2[2], b2[3])
func2(3, a2, b2)
print("   後 a2", a2[0], a2[1], a2[2], a2[3])
print("      b2", b2[0], b2[1], b2[2], b2[3])
			#
			# 2 次元配列
			#
print("2 次元配列")
a3 = [[1, 2, 3, 0], [4, 5, 6, 0]]
print("   前", a3[0][0], a3[0][1], a3[0][2], a3[0][3])
print("     ", a3[1][0], a3[1][1], a3[1][2], a3[1][3])
func3(2, 3, a3);
print("   後", a3[0][0], a3[0][1], a3[0][2], a3[0][3])
print("     ", a3[1][0], a3[1][1], a3[1][2], a3[1][3])
			#
			# クラスのインスタンス（オブジェクト）
			#
print("オブジェクト")
a4 = Example()
a4.init(1)
print("   前 n", a4.n)
print("      x ", a4.x[0], a4.x[1], a4.x[2])
func4(a4)
print("   後 n", a4.n)
print("      x ", a4.x[0], a4.x[1], a4.x[2])
			#
			# 関数名（sort）
			#
print("関数名（sort）")
a7 = [3, 2, 5, 4, 1]
a8 = [3, 2, 5, 4, 1]
print("   前", a7[0], a7[1], a7[2], a7[3], a7[4])
print("   前", a8[0], a8[1], a8[2], a8[3], a8[4])
func5(5, a7, ascend)   # 昇順
func5(5, a8, descend)   # 降順
print("   後", a7[0], a7[1], a7[2], a7[3], a7[4])
print("   後", a8[0], a8[1], a8[2], a8[3], a8[4])
			#
			# 関数名（Newton 法）
			#
print("関数名（Newton 法）")
eps1 = 1.0e-7
eps2 = 1.0e-10
x0   = 0.0
max  = 20
ind  = [0]
x    = func6(snx, dsnx, x0, eps1, eps2, max, ind)
print("   回数=", ind[0], "x=", x, "関数値=", snx(x), "微係数=", dsnx(x))
