############################
# 関数に対するプログラム例 #
#      coded by Y.Suganuma #
############################

##################################
# 2 つの整数の和                 #
#      u, v : 和を計算するデータ #
#      t1, t2, t3, t4 : 文字列   #
#      return : u + v            #
##################################
def func1(u, v, t1, t2, t3, t4)
	t1[0..0] = "x";
	t2[0..0] = "x";
	t3       = "ybc";
	t4       = "ybc";
	u        = 100;
	a        = u + v;
	return a;
end

########################
# 1 次元配列の要素の和 #
#      n : 要素数      #
#      u, v : 配列     #
########################
def func2(n, u)
	for i1 in 0 ... n
		u[n] += u[i1];
	end
end

########################
# 2 次元配列の要素の和 #
#      n : 行数        #
#      m : 列数        #
#      u : 配列        #
########################
def func3(n, m, u)
	for i1 in 0 ... n
		for i2 in 0 ... m
			u[i1][m] += u[i1][i2];
		end
	end
end

##################
# クラス Example #
##################
class Example
	def initialize(n)
		@_n = n;
		@_x = [1, 2, 3];
	end
	attr_accessor("_n", "_x");
end

########################
# オブジェクトの値変更 #
#      x1 : コピー     #
########################
def func4(x1)
	x1._n    = 4;
	x1._x[0] = 5;
end

#############################
# データの比較（昇順）      #
# a > bの場合に０以外（真） #
#############################
ascend = Proc.new { |a, b| a > b }

#############################
# データの比較（降順）      #
# a < bの場合に０以外（真） #
#############################
descend = Proc.new { |a, b|
	a < b;
}

#################################################################
# バブルソート                                                  #
#      n : データの数                                           #
#      data : データ                                            #
#      compare : 比較を行う関数                                 #
# バブルソートの手順                                            #
#　　　　　1)配列data[i]にn個のデータが入っているものとする．   #
#　　　　　　　　i=0,1,･･･,n-1                                  #
#　　　　　2)i番目のデータとi+1番目のデータを比較し，もし，     #
#　　　　　　　　data[i+1] < data[i]　（昇順の場合）            #
#　　　　　　であればそれらのデータを交換する．　i=0,1,･･･,n-2  #
#　　　　　　　　→この結果，data[n-1]に最も大きなデータが入る  #
#              ことになる                                       #
#　　　　　3)次に，n-1個のデータに対し上の処理を繰り返す．      #
#　　　　　4)同様に，n-2，n-3，・・・個のデータに対して同じ処理 #
#        を繰り返す                                             #
#################################################################
def func5(n, data, &compare)
	sw = 0;

	if n > 1 then
		i1 = 0;
		while i1 < n-1 do
			if compare.call(data[i1], data[i1+1]) then
				sw         = 1;
				x          = data[i1];
				data[i1]   = data[i1+1];
				data[i1+1] = x;
			end
			i1 += 1;
		end
	end

	if sw > 0 then
		func5(n-1, data, &compare);
	end
end

######################
# 関数値と微分の計算 #
######################
snx = Proc.new { |sw, x|
	if sw == 0 then
		Math.exp(x) - 3.0 * x;
	else
		Math.exp(x) - 3.0;
	end
}

#####################################################
# Newton法による非線形方程式(f(x)=0)の解            #
#      x0 : 初期値                                  #
#      eps1 : 終了条件１（｜x(k+1)-x(k)｜＜eps1）   #
#      eps2 : 終了条件２（｜f(x(k))｜＜eps2）       #
#      max : 最大試行回数                           #
#      ind : 実際の試行回数                         #
#            （負の時は解を得ることができなかった） #
#      fn : f(x)を計算する関数名                    #
#      dfn : f(x)の微分を計算する関数名             #
#      return : 解                                  #
#####################################################
def func6(x0, eps1, eps2, max, ind, &f)
	x      = x0;
	x1     = x0;
	sw     = 0;
	ind[0] = 0;

	while sw == 0 && ind[0] >= 0 do
		sw      = 1;
		ind[0] += 1;
		g       = f.call(0, x1);
		if g.abs() > eps2 then
			if ind[0] <= max then
				dg = f.call(1, x1);
				if dg.abs() > eps2 then
					x = x1 - g / dg;
					if (x-x1).abs() > eps1 && (x-x1).abs() > eps1*x.abs() then
						x1 = x;
						sw = 0;
					end
				else
					ind[0] = -1;
				end
			else
				ind[0] = -1;
			end
		end
	end

	return x;
end
			#
			# 基本型，文字列
			#
printf("基本型，文字列\n");
a1 = 10;
b1 = 20;
c1 = 0;
s1 = "abc";
s2 = String.new("abc");
s3 = "abc";
s4 = String.new("abc");
printf("   前 %d %d %d %s %s %s %s\n", a1, b1, c1, s1, s2, s3, s4);
c1 = func1(a1, b1, s1, s2, s3, s4);
printf("   前 %d %d %d %s %s %s %s\n", a1, b1, c1, s1, s2, s3, s4);
			#
			# 1 次元配列
			#
printf("1 次元配列\n");
a2 = [1, 2, 3, 0];   # a2 = Array[1, 2, 3, 0]; でも可
printf("   前 %d %d %d %d\n", a2[0], a2[1], a2[2], a2[3]);
func2(3, a2);
printf("   後 %d %d %d %d\n", a2[0], a2[1], a2[2], a2[3]);
			#
			# 2 次元配列
			#
printf("2 次元配列\n");
a3 = Array[[1, 2, 3, 0], [4, 5, 6, 0]];
printf("   前 %d %d %d %d\n", a3[0][0], a3[0][1], a3[0][2], a3[0][3]);
printf("      %d %d %d %d\n", a3[1][0], a3[1][1], a3[1][2], a3[1][3]);
func3(2, 3, a3);
printf("   後 %d %d %d %d\n", a3[0][0], a3[0][1], a3[0][2], a3[0][3]);
printf("      %d %d %d %d\n", a3[1][0], a3[1][1], a3[1][2], a3[1][3]);
			#
			# クラスのインスタンス（オブジェクト）
			#
printf("オブジェクト\n");
a4 = Example.new(1);
printf("   前 n %d\n", a4._n);
printf("      x %d %d %d\n", a4._x[0], a4._x[1], a4._x[2]);
func4(a4);
printf("   後 n %d\n", a4._n);
printf("      x %d %d %d\n", a4._x[0], a4._x[1], a4._x[2]);
			#
			# 関数名（sort）
			#
printf("関数名（sort）\n");
a7 = Array[3, 2, 5, 4, 1];
a8 = Array[3, 2, 5, 4, 1];
printf("   前 %d %d %d %d %d\n", a7[0], a7[1], a7[2], a7[3], a7[4]);
printf("   前 %d %d %d %d %d\n", a8[0], a8[1], a8[2], a8[3], a8[4]);
func5(5, a7, &ascend);   # 昇順
#func5(5, a7) { |a, b| a > b }   # 昇順
#func5(5, a7) do |a, b| a > b end   # 昇順
func5(5, a8, &descend);   # 降順
printf("   後 %d %d %d %d %d\n", a7[0], a7[1], a7[2], a7[3], a7[4]);
printf("   後 %d %d %d %d %d\n", a8[0], a8[1], a8[2], a8[3], a8[4]);
			#
			# 関数名（Newton 法）
			#
printf("関数名（Newton 法）\n");
eps1 = 1.0e-7;
eps2 = 1.0e-10;
x0   = 0.0;
max  = 20;
ind  = [0];
x    = func6(x0, eps1, eps2, max, ind, &snx);
printf("   回数=%d  x=%f  関数値=%f  微係数=%f\n", ind[0], x, snx.call(0, x), snx.call(1, x));
