/****************************/
/* back propagation model   */
/*      coded by Y.Suganuma */
/****************************/
using System;
using System.IO;

class Program
{
	/****************/
	/* main program */
	/****************/
	static void Main(String[] args)
	{
					// エラー
		if (args.Length != 2) {
			Console.WriteLine("***error   入力データファイル名を指定して下さい");
			Environment.Exit(1);
		}

		else {
					// ネットワークの定義
			Backpr net = new Backpr (args[0], args[1]);
					// 学習パターン等の入力
			Console.Write("学習回数は？ ");
			int m_tri = int.Parse(Console.ReadLine());   // 最大学習回数
			Console.Write("何回毎に収束を確認しますか？ ");
			int conv = int.Parse(Console.ReadLine());   // 収束確認回数

			Console.Write("学習パターンのファイル名は？ ");
			String f_name = Console.ReadLine();

			BackData dt1 = new BackData(f_name);
					// 学習
			int max = 0, no = 1;
			while (max < m_tri && no > 0) {

				int tri = ((max + conv) < m_tri) ? conv : m_tri - max;
				max    += tri;

				net.Learn(dt1, tri);   // 学習

				no = net.Recog(dt1, 0, max);   // 学習対象の認識

				Console.WriteLine("   回数 " + max + " 誤って認識したパターン数 " + no);
			}

			no = net.Recog(dt1, 1, max);   // 学習対象の認識と出力
					// 未学習パターンの認識
			Console.Write("未学習パターンの認識を行いますか？（=1:行う，=0:行わない） ");
			int sw = int.Parse(Console.ReadLine());

			if (sw > 0) {

				Console.Write("未学習パターンのファイル名は？ ");
				f_name = Console.ReadLine();

				BackData dt2 = new BackData(f_name);

				no = net.Recog(dt2, 2, max);   // 未学習対象の認識と出力
			}
		}
	}
}

/******************************************************/
/* バックプロパゲーションの制御（クラス BackControl） */
/******************************************************/
class BackControl {
	protected double alpha, eata;   //重み及びバイアス修正パラメータ
	protected double eps;   // 許容誤差
	protected int order;   // 入力パターンの与え方（=0:順番，=1:ランダム）
	protected int p_type;   // 出力先・方法の指定
                            //   =0 : 誤って認識した数だけ出力
                            //   =1 : 認識結果を出力
                            //   =2 : 認識結果と重みを出力
                            //        （負の時は，認識結果と重みをファイルへも出力）
	protected String o_file;   // 出力ファイル名

	/*************************************/
	/* クラスBackControlのコンストラクタ */
	/*      name : 入力データファイル名  */
	/*************************************/
	public BackControl(String name)
	{
		char[] charSep = new char[] {' '};
		String[] lines = File.ReadAllLines(name);
		String[] str   = lines[0].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		eps            = double.Parse(str[1]);
		p_type         = int.Parse(str[3]);
		if (p_type < 0)
			o_file = str[5];

		str   = lines[1].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		order = int.Parse(str[1]);
		eata  = double.Parse(str[3]);
		alpha = double.Parse(str[5]);
	}
}

/*****************************************************/
/* バックプロパゲーションのデータ（クラス BackData） */
/*****************************************************/
class BackData {
	public int noiu;   // 入力ユニットの数
	public int noou;   // 出力ユニットの数
	public int noip;   // 入力パターンの数
	public double[][] iptn;   // iptn[i][j] : (i+1)番目の入力パターンの(j+1)番目の
                              //              入力ユニットの入力値
                              //                i=0,noip-1  j=0,noiu-1
	public double[][] optn;   // optn[i][j] : (i+1)番目の入力パターンに対する(j+1)
                              //              番目の出力ユニットの目標出力値
                              //                i=0,noip-1  j=0,noou-1

	/************************************/
	/* クラスBackDataのコンストラクタ   */
	/*      name : 入力データファイル名 */
	/************************************/
	public BackData(String name)
	{
		char[] charSep = new char[] {' '};
		String[] lines = File.ReadAllLines(name);
				//
				// 入力パターン数等
				//
		String[] str = lines[0].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		noip         = int.Parse(str[1]);
		noiu         = int.Parse(str[3]);
		noou         = int.Parse(str[5]);
				//
				// 領域の確保
				//
		iptn = new double [noip][];
		optn = new double [noip][];
		for (int i1 = 0; i1 < noip; i1++) {
			iptn[i1] = new double [noiu];
			optn[i1] = new double [noou];
		}
				//
				// 入力パターン及び各入力パターンに対する出力パターンの入力
				//
		for (int i1 = 0; i1 < noip; i1++) {
			int k = 2 * i1 + 1;
			str = lines[k].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
			for (int i2 = 0; i2 < noiu; i2++)
				iptn[i1][i2] = double.Parse(str[i2+1]);
			str = lines[k+1].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
			for (int i2 = 0; i2 < noou; i2++)
				optn[i1][i2] = double.Parse(str[i2+1]);
		}
	}
}

/*******************************************/
/* バックプロパゲーション（クラス Backpr） */
/*******************************************/
class Backpr : BackControl {
	sbyte[][] con;   // con[i][j] : 各ユニットに対するバイアスの与え方，及び，
                     // 接続方法
                     //   [i][i] : ユニット(i+1)のバイアスの与え方
                     //     =-3 : 入力値で固定
                     //     =-2 : 入力値を初期値として，学習により変更
                     //     =-1 : 乱数で初期値を設定し，学習により変更
                     //   [i][j] : ユニット(i+1)と(j+1)の接続方法(j>i)
                     //     =0 : 接続しない
                     //     =1 : 接続する(重みの初期値を乱数で設定し，学習)
                     //     =2 : 接続する(重みの初期値を入力で与え，学習)
                     //     =3 : 接続する(重みを入力値に固定)
                     //            i=0,nou-1  j=0,nou-1
	int f_type;   // シグモイド関数のタイプ
                  //   0 : [0,1]
                  //   1 : [-1,1]）
	int noiu;   // 入力ユニットの数
	int noou;   // 出力ユニットの数
	int[] nohu;   // nohu[i] : レベル(i+1)の隠れ層のユニット数（隠れ層
                  //           には下から順に番号が付けられ，出力層はレ
                  //           ベル(nolvl+1)の隠れ層とも見做される）
                  //             i=0,nolvl
	int nolvl;   // 隠れユニットの階層数
	int nou;   // 入力，出力，及び，隠れ層の各ユニットの数の和（各ユニ
               // ットには最も上の出力ユニットから，隠れ層の各ユニット，
               // 及び，入力ユニットに至る一連のユニット番号が付けられ
               // る）
	double[] dp;   // dp[i] : ユニット(i+1)の誤差  i=0,nou-1
	double[] op;   // op[i] : ユニット(i+1)の出力  i=0,nou-1
	double[] theta;   //theta[i] : ユニット(i+1)のバイアス  i=0,nou
	double[][] w;   // w[i][j] : ユニット(i+1)から(j+1)への重み(j>i)
                    // w[j][i] : (i+1)から(j+1)への重みの前回修正量(j>i)
                    // w[i][i] : ユニット(i+1)のバイアスの前回修正量
                    //   i=0,nou-1  j=0,nou-1
	Random rn;   // 乱数

	/************************************************/
	/* クラスBackprのコンストラクタ                 */
	/*      name_c : 制御データ用入力ファイル名     */
	/*      name_s : ネットワーク記述入力ファイル名 */
	/************************************************/
	public Backpr(String name_c, String name_s) : base(name_c)
	{
		char[] charSep = new char[] {' '};
		String[] lines = File.ReadAllLines(name_s);
				//
				// 乱数の初期化
				//
		rn  = new Random();   // 乱数の初期設定
				//
				// 入力ユニット，出力ユニットの数，関数タイプ
				//
		String[] str = lines[0].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		noiu         = int.Parse(str[1]);
		noou         = int.Parse(str[3]);
		f_type       = int.Parse(str[5]);
		nou          = noiu + noou;   // 入力ユニットと出力ユニットの和
				//
				// 隠れユニットの階層数と各階層のユニット数
				//
		str         = lines[1].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		nolvl       = int.Parse(str[1]);
		nohu        = new int [nolvl+1];
		nohu[nolvl] = noou;   // 出力ユニットの数

		if (nolvl > 0) {
			for (int i1 = 0; i1 < nolvl; i1++) {
				nohu[i1]  = int.Parse(str[i1+3]);
				nou      += nohu[i1];
			}
		}
				//
				// 領域の確保
				//
		con = new sbyte [nou][];
		for (int i1 = 0; i1 < nou; i1++) {
			con[i1] = new sbyte [nou];
			for (int i2 = 0; i2 < nou; i2++)
				con[i1][i2] =  (i1 == i2) ? (sbyte)-1 : (sbyte)0;
		}

		w = new double [nou][];
		for (int i1 = 0; i1 < nou; i1++) {
			w[i1] = new double [nou];
			for (int i2 = 0; i2 < nou; i2++)
				w[i1][i2] = 0.0;
		}

		dp    = new double [nou];
		op    = new double [nou];
		theta = new double [nou];

		for (int i1 = 0; i1 < nou; i1++)
			theta[i1] = 0.2 * rn.NextDouble() - 0.1;
				//
				// 各ユニットのバイアスとユニット間の接続関係
				//
					// バイアス
						// バイアスデータの数
		str    = lines[2].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		int nb = int.Parse(str[1]);
		if (nb > 0) {
						// バイアスデータの処理
			for (int i0 = 0; i0 < nb; i0++) {
							// ユニット番号
				str    = lines[i0+6].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
				int k1 = int.Parse(str[0]);
							// 不適当なユニット番号のチェック
				if (k1 < 1 || k1 > (nou-noiu)) {
					Console.WriteLine("***error  ユニット番号 " + k1 + " が不適当");
					Environment.Exit(1);
				}
							// バイアスの与え方
				k1--;
				int id      = int.Parse(str[1]);
				con[k1][k1] = (sbyte)id;
							// バイアスの初期設定
				switch (con[k1][k1]) {
					case -1:
						double x1 = double.Parse(str[2]);
						double x2 = double.Parse(str[3]);
						theta[k1] = (x2 - x1) * rn.NextDouble() + x1;
						break;
					case -2:
						theta[k1] = double.Parse(str[2]);
						break;
					case -3:
						theta[k1] = double.Parse(str[2]);
						break;
					default:
						Console.WriteLine("***error  バイアスの与え方が不適当");
						Environment.Exit(1);
						break;
				}
			}
		}
					// 接続方法
						// 接続データの数
		str   = lines[nb+6].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		int n = int.Parse(str[1]);

		if (n > 0) {
						// 接続データの処理
			for (int i0 = 0; i0 < n; i0++) {
							// 接続情報
				str    = lines[nb+10+i0].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
				int k1 = int.Parse(str[0]);
				int k2 = int.Parse(str[1]);
				int k3 = int.Parse(str[2]);
				int k4 = int.Parse(str[3]);
							// 不適切な接続のチェック
				int sw = 0;
				if (k1 < 1 || k2 < 1 || k3 < 1 || k4 < 1)
					sw = 1;
				else {
					if (k1 > nou || k2 > nou || k3 > nou || k4 > nou)
						sw = 1;
					else {
						if (k1 > k2 || k3 > k4)
							sw = 1;
						else {
							if (k4 >= k1)
								sw = 1;
							else {
								int l1 = -1;
								int k  = 0;
								for (int i1 = nolvl; i1 >= 0 && l1 < 0; i1--) {
									k += nohu[i1];
									if (k1 <= k)
										l1 = i1;
								}
								int l2 = -1;
								k      = 0;
								for (int i1 = nolvl; i1 >= 0 && l2 < 0; i1--) {
									k += nohu[i1];
									if (k4 <= k)
										l2 = i1;
								}
								if (l2 <= l1)
									sw = 1;
							}
						}
					}
				}

				if (sw > 0) {
					Console.WriteLine("***error  ユニット番号が不適当（"
                                       + k1 + " " + k2 + " " + k3 + " " + k4 + "）");
					Environment.Exit(1);
				}
							// 重みの初期値の与え方
				k1--;
				k2--;
				k3--;
				k4--;

				int id    = int.Parse(str[4]);
				double x1 = 0.0;
				double x2 = 0.0;

				if (id == 1) {
					x1 = double.Parse(str[5]);
					x2 = double.Parse(str[6]);
				}
				else {
					if (id > 1)
						x1 = double.Parse(str[5]);
					else {
						if (id != 0) {
							Console.WriteLine("***error  接続方法が不適当");
							Environment.Exit(1);
						}
					}
				}
							// 重みの初期値の設定
				for (int i1 = k3; i1 <= k4; i1++) {
					for (int i2 = k1; i2 <= k2; i2++) {
						con[i1][i2] = (sbyte)id;
						switch (id) {
							case 0:
								w[i1][i2] = 0.0;
								break;
							case 1:
								w[i1][i2] = (x2 - x1) * rn.NextDouble() + x1;
								break;
							case 2:
								w[i1][i2] = x1;
								break;
							case 3:
								w[i1][i2] = x1;
								break;
						}
					}
				}
			}
		}
	}

	/******************************************/
	/* 誤差の計算，及び，重みとバイアスの修正 */
	/*      ptn[i1] : 出力パターン            */
	/******************************************/
	void Err_back(double[] ptn)
	{
		for (int i1 = 0; i1 < nou-noiu; i1++) {
					// 誤差の計算
			if (i1 < noou) {
				if (f_type == 0)
					dp[i1] = (ptn[i1] - op[i1]) * op[i1] * (1.0 - op[i1]);
				else
					dp[i1] = 0.5 * (ptn[i1] - op[i1]) * 
                             (op[i1] - 1.0) * (op[i1] + 1.0);
			}
			else {
				double x1 = 0.0;
				for (int i2 = 0; i2 < i1; i2++) {
					if (con[i2][i1] > 0)
						x1 += dp[i2] * w[i2][i1];
				}
				if (f_type == 0)
					dp[i1] = op[i1] * (1.0 - op[i1]) * x1;
				else
					dp[i1] = 0.5 * (op[i1] - 1.0) * (op[i1] + 1.0) * x1;
			}
					// 重みの修正
			for (int i2 = i1+1; i2 < nou; i2++) {
				if (con[i1][i2] == 1 || con[i1][i2] == 2) {
					double x1  = eata * dp[i1] * op[i2] + alpha * w[i2][i1];
					w[i2][i1]  = x1;
					w[i1][i2] += x1;
				}
			}
					// バイアスの修正
			if (con[i1][i1] >= -2) {
				double x1  = eata * dp[i1] + alpha * w[i1][i1];
				w[i1][i1]  = x1;
				theta[i1] += x1;
			}
		}
	}

	/********************************************************/
	/* 与えられた入力パターンに対する各ユニットの出力の計算 */
	/********************************************************/
	void Forward()
	{
		for (int i1 = nou-noiu-1; i1 >= 0; i1--) {

			double sum = -theta[i1];

			for (int i2 = i1+1; i2 < nou; i2++) {
				if (con[i1][i2] > 0)
					sum -= w[i1][i2] * op[i2];
			}

			op[i1] = (f_type == 0) ? 1.0 / (1.0 + Math.Exp(sum)) :
                                     1.0 - 2.0 / (1.0 + Math.Exp(sum));
		}
	}

	/*****************************/
	/* 学習の実行                */
	/*      p : 認識パターン     */
	/*      m_tri : 最大学習回数 */
	/*****************************/
	public void Learn(BackData p, int m_tri)
	{
				//
				// エラーチェック
				//
		if (noiu != p.noiu || noou != p.noou) {
			Console.WriteLine("***error  入力または出力ユニットの数が違います");
			Environment.Exit(1);
		}

		int k0 = -1;
		for (int i1 = 0; i1 < m_tri; i1++) {
				//
				// パターンを与える順番の決定
				//
			if (order == 0) {       // 順番
				k0++;
				if (k0 >= p.noip)
					k0 = 0;
			}
			else {                  // ランダム
				k0 = (int)(rn.NextDouble() * p.noip);
				if (k0 >= p.noip)
					k0 = p.noip - 1;
			}
				//
				// 出力ユニットの結果を計算
				//
			int k1 = nou - noiu;
			for (int i2 = 0; i2 < noiu; i2++)
				op[k1+i2] = p.iptn[k0][i2];

			Forward();
				//
				// 重みとバイアスの修正
				//
			Err_back(p.optn[k0]);
		}
	}

	/***********************************************/
	/* 与えられた対象の認識と出力                  */
	/*      p : 認識パターン                       */
	/*      pr : =0 : 出力を行わない               */
	/*           =1 : 出力を行う                   */
	/*           =2 : 出力を行う（未学習パターン） */
	/*      tri : 現在の学習回数                   */
	/*      return : 誤って認識したパターンの数    */
	/***********************************************/
	public int Recog(BackData p, int pr, int tri)
	{
				//
				// ファイルのオープン
				//
		StreamWriter OUT = null;
		if (p_type < 0 && pr > 0) {
			OUT = new StreamWriter(o_file, true); 
			if (pr == 1) {
				OUT.WriteLine("***学習パターン***");
				OUT.WriteLine();
			}
			else {
				OUT.WriteLine();
				OUT.WriteLine("***未学習パターン***");
				OUT.WriteLine();
			}
		}
				//
				// 各パターンに対する出力
				//
		int No = 0;

		for (int i1 = 0; i1 < p.noip; i1++) {
					// 入力パターンの設定
			int k1 = nou - noiu;
			for (int i2 = 0; i2 < noiu; i2++)
				op[k1+i2] = p.iptn[i1][i2];
					// 出力の計算
			Forward();
					// 結果の表示
			if (p_type != 0 && pr > 0) {

				Console.Write("入力パターン" + (i1+1) + "   ");
				for (int i2 = 0; i2 < noiu; i2++) {
					Console.Write(" " + op[k1+i2]);
					if (i2 == noiu-1)
						Console.WriteLine();
					else {
						if(((i2+1) % 10) == 0) {
							Console.WriteLine();
							Console.Write("                   ");
						}
					}
				}

				Console.WriteLine();
				Console.Write("    出力パターン(理想)  ");
				for (int i2 = 0; i2 < noou; i2++) {
					Console.Write(" " + p.optn[i1][i2]);
					if (i2 == noou-1)
						Console.WriteLine();
					else {
						if(((i2+1) % 5) == 0) {
							Console.WriteLine();
							Console.Write("                        ");
						}
					}
				}
			}

			int sw = 0;
			if (p_type != 0 && pr > 0)
				Console.Write("                (実際)  ");
			for (int i2 = 0; i2 < noou; i2++) {
				if (p_type != 0 && pr > 0) {
					Console.Write(" " + op[i2]);
					if (i2 == noou-1)
						Console.WriteLine();
					else {
						if(((i2+1) % 5) == 0) {
							Console.WriteLine();
							Console.Write("                        ");
						}
					}
				}
				if (Math.Abs(op[i2]-p.optn[i1][i2]) > eps)
					sw = 1;
			}

			if (sw > 0)
				No++;

			if (p_type < 0 && pr > 0) {

				OUT.Write("入力パターン" + (i1+1) + "   ");
				for (int i2 = 0; i2 < noiu; i2++) {
					OUT.Write(" " + op[k1+i2]);
					if (i2 == noiu-1)
						OUT.WriteLine();
					else {
						if(((i2+1) % 10) == 0) {
							OUT.WriteLine();
							OUT.Write("                   ");
						}
					}
				}

				OUT.WriteLine();
				OUT.Write("    出力パターン(理想)  ");
				for (int i2 = 0; i2 < noou; i2++) {
					OUT.Write(" " + p.optn[i1][i2]);
					if (i2 == noou-1)
						OUT.WriteLine();
					else {
						if(((i2+1) % 5) == 0) {
							OUT.WriteLine();
							OUT.Write("                        ");
						}
					}
				}

				OUT.Write("                (実際)  ");
				for (int i2 = 0; i2 < noou; i2++) {
					OUT.Write(" " + op[i2]);
					if (i2 == noou-1)
						OUT.WriteLine();
					else {
						if(((i2+1) % 5) == 0) {
							OUT.WriteLine();
							OUT.Write("                        ");
						}
					}
				}
			}

			if (p_type != 0 && pr > 0)
				Console.ReadLine();
		}
				//
				// 重みの出力
				//
		int ln;
		if ((p_type < -1 || p_type > 1) && pr == 1) {

			Console.WriteLine("    重み");
			for (int i1 = 0; i1 < nou-noiu; i1++) {
				Console.Write("      to " + (i1+1) + " from  ");
				ln = -1;
				for (int i2 = 0; i2 < nou; i2++) {
					if (con[i1][i2] > 0) {
						if (ln <= 0) {
							if (ln < 0)
								ln = 0;
							else {
								Console.WriteLine();
								Console.Write("                   ");
							}
						}
						Console.Write(" " + (i2+1) + " " + w[i1][i2]);
						ln += 1;
						if (ln == 4)
							ln = 0;
					}
				}

				Console.WriteLine();
			}

			Console.WriteLine();
			Console.Write("    バイアス  ");
			ln = 0;
			for (int i1 = 0; i1 < nou-noiu; i1++) {
				Console.Write(" " + (i1+1) + " " + theta[i1]);
				ln += 1;
				if (ln == 4 && i1 != nou-noiu-1) {
					ln = 0;
					Console.WriteLine();
					Console.Write("              ");
				}
			}

			if (ln != 0)
				Console.WriteLine();

			Console.ReadLine();
		}

		if (p_type < 0 && pr == 1) {

			OUT.WriteLine("    重み");
			for (int i1 = 0; i1 < nou-noiu; i1++) {
				OUT.Write("      to " + (i1+1) + " from  ");
				ln = -1;
				for (int i2 = 0; i2 < nou; i2++) {
					if (con[i1][i2] > 0) {
						if (ln <= 0) {
							if (ln < 0)
								ln = 0;
							else {
								OUT.WriteLine();
								OUT.Write("                   ");
							}
						}
						OUT.Write(" " + (i2+1) + " " + w[i1][i2]);
						ln += 1;
						if (ln == 4)
							ln = 0;
					}
				}

				OUT.WriteLine();
			}

			OUT.WriteLine();
			OUT.Write("    バイアス  ");
			ln = 0;
			for (int i1 = 0; i1 < nou-noiu; i1++) {
				OUT.Write(" " + (i1+1) + " " + theta[i1]);
				ln += 1;
				if (ln == 4 && i1 != nou-noiu-1) {
					ln = 0;
					OUT.WriteLine();
					OUT.Write("\n              ");
				}
			}

			if (ln != 0)
				OUT.WriteLine();
		}

		if (p_type < 0 && pr > 0)
			OUT.Close();

		return No;
	}
}

/*
------------------------制御データ----------------
誤差 0.1 出力 -2 出力ファイル kekka
順番 0 η 0.5 α 0.8
画面表示（円の大きさ，フォントサイズ，幅，高さ） 20 20 400 300

------------------------構造データ----------------
入力ユニット数 2 出力ユニット数 1 関数タイプ 0
隠れ層の数 1 各隠れ層のユニット数（下から） 1
バイアス入力ユニット数 1
　ユニット番号：出力ユニットから順に番号付け
　入力方法：=-3:固定，=-2:入力後学習，=-1:乱数(default，[-0.1,0.1])）
　値：バイアス値（ー２またはー３の時）または一様乱数の範囲（下限，上限）
1 -1 -0.05 0.05
接続方法の数 2
　ユニット番号：ユニットk1からk2を，k3からk4に接続
　接続方法：=0:接続なし，=1:乱数，=2:重み入力後学習，=3:重み固定
　値：重み（２または３の時）または一様乱数の範囲（１の時：下限，上限）
3 4 1 2 1 -0.1 0.1
2 2 1 1 1 -0.1 0.1

------------------------学習データ----------------
パターンの数 4 入力ユニット数 2 出力ユニット数 1
入力１ 0 0
　出力１ 0
入力２ 0 1
　出力２ 1
入力３ 1 0
　出力３ 1
入力４ 1 1
　出力４ 0

------------------------認識データ----------------
パターンの数 4 入力ユニット数 2 出力ユニット数 1
入力１ 0 0
　出力１ 0
入力２ 0 1
　出力２ 1
入力３ 1 0
　出力３ 1
入力４ 1 1
　出力４ 0
*/
