# -*- coding: UTF-8 -*-
import numpy as np
from math import *

############################################
# secant法（はさみうち法）による非線形方程式f(x)=0の解
#      x1,x2 : 初期値（x1 < x2）
#      max : 最大試行回数
#      eps1 : 終了条件１（｜x(k+1)-x(k)｜＜eps1）
#      eps2 : 終了条件２（｜f(x(k))｜＜eps2）
#      ind : > 0 : 収束回数
#            =-1 : 収束しなかった
#      fn : f(x)を計算する関数名
#      return : 解
#      coded by Y.Suganuma
############################################

def secant(x1, x2, max, eps1, eps2, ind, fn) :

	x      = 0.0
	count  = 0
	ind[0] = 0
	f1     = fn(x1)
	f2     = fn(x2)

	if abs(f1) < eps2 :
		x = x1
	else :
		if abs(f2) < eps2 :
			x = x2
		else :
			while ind[0] == 0 :
				count += 1
				if abs(f2-f1) < eps2 :
					ind[0] = -1
				else :
					x = x2 - f2 * (x2 - x1) / (f2 - f1)
					f = fn(x)
					if abs(f) < eps2 or abs(x2-x1) < eps1 or abs(x2-x1) < eps1*abs(x2) :
						ind[0] = count
					else :
						if count < max :
							if f1*f2 < 0.0 :
								x2 = x
								f2 = f
							else :
								x1 = x
								f1 = f
						else :
							ind[0] = -1

	return x

################
# 関数値の計算 #
################
def snx(x) :
	return exp(x) - 3.0 * x

############################################
# セカント法による exp(x)-3x=0 の根
#      coded by Y.Suganuma
############################################

			# データの設定
ind  = [0];
eps1 = 1.0e-10
eps2 = 1.0e-10
max  = 100
x1   = 0.0
x2   = 1.0
			# 実行と結果
x = secant(x1, x2, max, eps1, eps2, ind, snx)

print("   ind=" + str(ind[0]) + "  x=" + str(x) + "  f= " + str(snx(x)))
