# -*- coding: UTF-8 -*-
import numpy as np
from math import *

#*********************************/
# ３次スプライン関数による補間   */
#      n : 区間の数              */
#      x, y : 点の座標           */
#      x1 : 補間値を求める値     */
#      h, b, d, g, u, r : 作業域 */
#      return : 補間値           */
#      coded by Y.Suganuma       */
#*********************************/
def spline(n, x, y, x1, h, b, d, g, u, r) :
					# 区間の決定
	i = -1
	for i1 in range(1, n) :
		if x1 < x[i1] :
			i = i1 - 1
		if i >= 0 :
			break
	if i < 0 :
		i = n - 1
					# ステップ１
	for i1 in range(0, n) :
		h[i1] = x[i1+1] - x[i1]
	for i1 in range(1, n) :
		b[i1] = 2.0 * (h[i1] + h[i1-1])
		d[i1] = 3.0 * ((y[i1+1] - y[i1]) / h[i1] - (y[i1] - y[i1-1]) / h[i1-1])
					# ステップ２
	g[1] = h[1] / b[1]
	for i1 in range(2, n-1) :
		g[i1] = h[i1] / (b[i1] - h[i1-1] * g[i1-1])
	u[1] = d[1] / b[1]
	for i1 in range(2, n) :
		u[i1] = (d[i1] - h[i1-1] * u[i1-1]) / (b[i1] - h[i1-1] * g[i1-1])
					# ステップ３
	if i1 > 1 :
		k = i
	else :
		k = 1
	r[0]   = 0.0
	r[n]   = 0.0
	r[n-1] = u[n-1]
	for i1 in range(n-2, k-1, -1) :
		r[i1] = u[i1] - g[i1] * r[i1+1]
					# ステップ４
	xx = x1 - x[i]
	qi = (y[i+1] - y[i]) / h[i] - h[i] * (r[i+1] + 2.0 * r[i]) / 3.0
	si = (r[i+1] - r[i]) / (3.0 * h[i])
	y1 = y[i] + xx * (qi + xx * (r[i]  + si * xx))

	return y1

#*******************************/
# 補間法（３次スプライン関数） */
#      coded by Y.Suganuma     */
#*******************************/
					# 設定
n = 5
x = np.array([0, 1, 2, 3, 4, 5], np.float)
y = np.array([0, 1, 4, 9, 16, 25], np.float)
h = np.empty(n, np.float)
b = np.empty(n, np.float)
d = np.empty(n, np.float)
g = np.empty(n, np.float)
u = np.empty(n, np.float)
r = np.empty(n+1, np.float)
					# 実行と出力
x1 = 0.0
sp = 0.1
while x1 <= 5.01 :
	y1 = spline(n, x, y, x1, h, b, d, g, u, r)
	print(x1, y1)
	x1 += sp
