#*******************************/
# 補間法（３次スプライン関数） */
#      coded by Y.Suganuma     */
#*******************************/

#***************/
# クラスの定義 */
#***************/
class Spline

	#*************************/
	# コンストラクタ         */
	#      n1 : 区間の数     */
	#      x1, y1 : 点の座標 */
	#*************************/
	def initialize(n1, x1, y1)
					# 領域の確保
		h = Array.new(n1)
		b = Array.new(n1)
		d = Array.new(n1)
		g = Array.new(n1)
		u = Array.new(n1)
		@_n = n1
		@_q = Array.new(@_n)
		@_s = Array.new(@_n)
		@_r = Array.new(@_n+1)
		@_x = Array.new(@_n+1)
		@_y = Array.new(@_n+1)

		for i1 in 0 ... @_n+1
			@_x[i1] = x1[i1]
			@_y[i1] = y1[i1]
		end
					# ステップ１
		for i1 in 0 ... @_n
			h[i1] = @_x[i1+1] - @_x[i1]
		end
		for i1 in 1 ... @_n
			b[i1] = 2.0 * (h[i1] + h[i1-1])
			d[i1] = 3.0 * ((@_y[i1+1] - @_y[i1]) / h[i1] - (@_y[i1] - @_y[i1-1]) / h[i1-1])
		end
					# ステップ２
		g[1] = h[1] / b[1]
		for i1 in 2 ... @_n-1
			g[i1] = h[i1] / (b[i1] - h[i1-1] * g[i1-1])
		end
		u[1] = d[1] / b[1]
		for i1 in 2 ... @_n
			u[i1] = (d[i1] - h[i1-1] * u[i1-1]) / (b[i1] - h[i1-1] * g[i1-1])
		end
					# ステップ３
		@_r[0]     = 0.0
		@_r[@_n]   = 0.0
		@_r[@_n-1] = u[@_n-1]
		i1         = @_n - 2
		while i1 >= 1
			@_r[i1] = u[i1] - g[i1] * @_r[i1+1]
			i1     -= 1
		end
					# ステップ４
		for i1 in 0 ... @_n
			@_q[i1] = (@_y[i1+1] - @_y[i1]) / h[i1] - h[i1] * (@_r[i1+1] + 2.0 * @_r[i1]) / 3.0
			@_s[i1] = (@_r[i1+1] - @_r[i1]) / (3.0 * h[i1])
		end
	end

	#*****************************/
	# 補間値の計算               */
	#      x1 : 補間値を求める値 */
	#      return : 補間値       */
	#*****************************/
	def value(x1)
					# 区間の決定
		i = -1
		for i1 in 1 ... @_n
			if (x1 < @_x[i1])
				i = i1 - 1
			end
			if i >= 0
				break
			end
		end
		if (i < 0)
			i = @_n - 1
		end
					# 計算
		xx = x1 - @_x[i]
		y1 = @_y[i] + xx * (@_q[i] + xx * (@_r[i]  + @_s[i] * xx))

		return y1
	end
end

					# 設定
n = 5
x = Array[0.0, 1.0, 2.0, 3.0, 4.0, 5.0]
y = Array[0.0, 1.0, 4.0, 9.0, 16.0, 25.0]

spline = Spline.new(n, x, y)
				# 実行と出力
x1 = 0.0
sp = 0.1
while x1 <= 5.01
	y1 = spline.value(x1)
	printf("%f %f\n", x1, y1)
	x1 += sp
end
