# -*- coding: UTF-8 -*-
import numpy as np
import sys
from math import *

"""  入力データ例
投資先 3 投資単位 1 ケース数 5
以下，各投資先に資金を投資した場合の利益を入力（ケース数だけ入力）
投資先１ 0 8 18 22 24
投資先２ 0 3 6 9 12
投資先３ 0 6 7 8 10
"""

############################################
# 動的計画法によるナップサック問題
#      p : 現在計算中の投資先
#      n_toshi : 投資先の数
#      step : 資金の投資単位
#      n_case : 資金の投資ケース数
#      table : 投資先，投資金額毎の利益
#      rieki : 現段階の最大利益（φ）
#      next : 最大利益を与える投資先
#      coded by Y.Suganuma
############################################

def dynamic(p, n_toshi, step, n_case, table, rieki, next) :

	m = 0
			# 最大利益の計算
				# １段目
	if p == 0 :
		for i1 in range(0, n_case) :
			rieki[p][i1] = table[p][i1]
				# ２段目以降
	else :
		for i1 in range(0, n_case) :
			l1  = -1
			l2  = -1
			max = -1
			m   = step * i1
			m1  = 0
			k1  = 0
			while m1 <= m :
				m2 = 0
				k2 = 0
				while m1+m2 <= m :
					r = table[p][k1] + rieki[p-1][k2]
					if r > max :
						l1  = k1
						l2  = k2
						max = r
					m2 += step
					k2 += 1
				m1 += step
				k1 += 1
			next[p][i1] = l2
			if l2 >= 0 :
				rieki[p][i1] = table[p][l1] + rieki[p-1][l2]
			# 次の投資先
				# 最終段より前
	if p < n_toshi-1 :
		m = dynamic(p+1, n_toshi, step, n_case, table, rieki, next)
				# 最終段
	else :
		max = -1
		k1  = n_toshi - 1
		for i1 in range(0, n_case) :
			if next[k1][i1] >= 0 and rieki[k1][i1] > max :
				max = rieki[k1][i1]
				m   = i1

	return m

############################################
# 動的計画法（ナップサック問題）
#      coded by Y.Suganuma
############################################

			# データの入力
line    = sys.stdin.readline()
s       = line.split()
n_toshi = int(s[1])
step    = int(s[3])
n_case  = int(s[5])

line    = sys.stdin.readline()

table   = np.empty((n_toshi, n_case), np.int)
rieki   = np.empty((n_toshi, n_case), np.int)
next    = np.empty((n_toshi, n_case), np.int)
for i1 in range(0, n_toshi) :
	line = sys.stdin.readline()
	s    = line.split()
	for i2 in range(0, n_case) :
		table[i1][i2] = int(s[i2+1])
			# 実行
max = dynamic(0, n_toshi, step, n_case, table, rieki, next)
			# 結果の出力
if max >= 0 :
	print("最大利益 " + str(rieki[n_toshi-1][max]))
	for i1 in range(n_toshi-1, -1, -1) :
		k = max * step
		if i1 > 0 :
			max  = next[i1][max]
			k   -= max * step
		print("   投資先 " + str(i1+1) + " 投資資金 " + str(k))
else :
	print("解が存在しません！")
