/********************************/
/* 伝達関数のゲインと位相の計算 */
/*      coded by Y.Suganuma     */
/********************************/
using System;
using System.IO;

class Program
{
	/****************/
	/* main program */
	/****************/
	static void Main(String[] args)
	{
		double uc      = 90.0 / Math.Asin(1.0);
		char[] charSep = new char[] {' '};
				//
				// データの入力
				//
					// 出力ファイル名とファイルのオープン
		String[] str = Console.ReadLine().Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		StreamWriter g_o = new StreamWriter(str[1]);
		StreamWriter p_o = new StreamWriter(str[3]);
					// 伝達関数データ
		str       = Console.ReadLine().Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		double fl = double.Parse(str[1]);
		double fu = double.Parse(str[2]);
		int k     = int.Parse(str[4]);
						// 分子
		str    = Console.ReadLine().Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		int ms = int.Parse(str[1]);
		int m  = int.Parse(str[3]);
							// 因数分解されていない
		double[] si;
		if (ms == 0) {
			int k1 = m + 1;
			si     = new double [k1];
			for (int i1 = 0; i1 < k1; i1++)
				si[i1] = double.Parse(str[i1+5]);
		}
							// 因数分解されている
		else {
			int k1 = (m == 0) ? 1 : 2 * m;
			si     = new double [k1];
			for (int i1 = 0; i1 < k1; i1++)
				si[i1] = double.Parse(str[i1+5]);
		}
						// 分母
		str    = Console.ReadLine().Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		int mb = int.Parse(str[1]);
		int n  = int.Parse(str[3]);
							// 因数分解されていない
		double[] bo;
		if (mb == 0) {
			int k1 = n + 1;
			bo = new double [k1];
			for (int i1 = 0; i1 < k1; i1++)
				bo[i1] = double.Parse(str[i1+5]);
		}
							// 因数分解されている
		else {
			int k1 = (n == 0) ? 1 : 2 * n;
			bo     = new double [k1];
			for (int i1 = 0; i1 < k1; i1++)
				bo[i1] = double.Parse(str[i1+5]);
		}
				//
				// インと位相の計算
				//
		double h  = (Math.Log10(fu) - Math.Log10(fl)) / k;
		double ff = Math.Log10(fl);

		double phase = 0.0;
		for (int i1 = 0; i1 <= k; i1++) {
					// 周波数の対数を元に戻す
			double f = Math.Pow(10.0, ff);
					// 値の計算
			Complex g = G_s(f, ms, m, si, mb, n, bo);
					// ゲインと位相の計算
			double gain = 20.0 * Math.Log10(Complex.abs(g));
			double x    = Complex.angle(g) * uc;
			while (Math.Abs(phase-x) > 180.0) {
				if (x-phase > 180.0)
					x -= 360.0;
				else {
					if (x-phase < -180.0)
						x += 360.0;
				}
			}
			phase = x;
					// 出力
			g_o.WriteLine(f + " " + gain);
			p_o.WriteLine(f + " " + phase);
					// 次の周波数
			ff += h;
		}

		g_o.Close();
		p_o.Close();
	}

	/********************************************************************/
	/* 伝達関数のｓにｊωを代入した値の計算                             */
	/*      ff : ω（周波数）                                           */
	/*      ms : 分子の表現方法                                         */
	/*             =0 : 多項式（si[0]+si[1]*s+si[1]*s^2+･･･）           */
	/*             =1 : 因数分解（(si[0]+si[1]*s)*(si[2]+si[3]*s)*･･･） */
	/*      m : 分子の次数                                              */
	/*      si : 分子多項式の係数                                       */
	/*      mb : 分母の表現方法                                         */
	/*             =0 : 多項式（bo[0]+bo[1]*s+bo[1]*s^2+･･･）           */
	/*             =1 : 因数分解（(bo[0]+bo[1]*s)*(bo[2]+bo[3]*s)*･･･） */
	/*      n : 分母の次数                                              */
	/*      bo : 分母多項式の係数                                       */
	/*      return : 結果                                               */
	/********************************************************************/
	static Complex G_s(double ff, int ms, int m, double[] si, int mb, int n, double[] bo)
	{
					// 周波数を複素数に変換
		Complex f = new Complex (0.0, ff);
					// 分子
		Complex x = value(f, ms, m, si);
					// 分母
		Complex y = value(f, mb, n, bo);

		return Complex.dev(x, y);
	}

	/****************************************************************/
	/* 多項式のｓにｊωを代入した値の計算                           */
	/*      f : ｊω（周波数，複素数）                              */
	/*      ms : 多項式の表現方法                                   */
	/*             =0 : 多項式（a[0]+a[1]*s+a[1]*s^2+･･･）          */
	/*             =1 : 因数分解（(a[0]+a[1]*s)*(a[2]+a[3]*s)*･･･） */
	/*      n : 多項式の次数                                        */
	/*      a : 多項式の係数                                        */
	/*      return : 結果                                           */
	/****************************************************************/
	static Complex value(Complex f, int ms, int n, double[] a)
	{
		Complex x;
					// 因数分解されていない
		if (ms == 0) {
			x = new Complex (0.0, 0.0);
			for (int i1 = 0; i1 <= n; i1++)
				x = Complex.add(x, Complex.mul(new Complex(a[i1]), Complex.pow(f, i1)));
		}
					// 因数分解されている
		else {
			if (n == 0)
				x = new Complex (a[0]);
			else {
				x      = new Complex (1.0, 0.0);
				int k1 = 2 * n;
				for (int i1 = 0; i1 < k1; i1 += 2)
					x = Complex.mul(x, Complex.add(new Complex(a[i1]), Complex.mul(new Complex(a[i1+1]), f)));
			}
		}

		return x;
	}
}

/****************************/
/* クラスComplex            */
/*      coded by Y.Suganuma */
/****************************/
class Complex {
	double r;   // 実部
	double i;   // 虚部

	/******************/
	/* コンストラクタ */
	/*      a : 実部  */
	/*      b : 虚部  */
	/******************/
	public Complex(double a = 0.0, double b = 0.0)
	{
		r = a;
		i = b;
	}

	/********/
	/* 出力 */
	/********/
	public void output(StreamWriter OUT, Complex a)
	{
		OUT.Write("(" + a.r + ", " + a.i + ")");
	}

	/******************/
	/* 複素数の絶対値 */
	/******************/
	public static double abs(Complex a)
	{
		return Math.Sqrt(a.r * a.r + a.i * a.i);
	}

	/****************/
	/* 複素数の角度 */
	/****************/
	public static double angle(Complex a)
	{
		double x = 0.0;
		if (a.r != 0.0 || a.i != 0.0)
			x = Math.Atan2(a.i, a.r);
		return x;
	}

	/****************/
	/* 複素数のｎ乗 */
	/****************/
	public static Complex pow(Complex a, int n)
	{
		Complex c = new Complex (1);
		if (n >= 0) {
			for (int i1 = 0; i1 < n; i1++)
				c = Complex.mul(c, a);
		}
		else {
			for (int i1 = 0; i1 < -n; i1++)
				c = Complex.mul(c, a);
			c = Complex.dev(new Complex(1.0), c);
		}
		return c;
	}

	/****************/
	/* 複素数の加算 */
	/****************/
	public static Complex add(Complex a, Complex b)
	{
		Complex c = new Complex();
		c.r = a.r + b.r;
		c.i = a.i + b.i;
		return c;
	}

	/****************/
	/* 複素数の減算 */
	/****************/
	public static Complex sub(Complex a, Complex b)
	{
		Complex c = new Complex();
		c.r = a.r - b.r;
		c.i = a.i - b.i;
		return c;
	}

	/****************/
	/* 複素数の乗算 */
	/****************/
	public static Complex mul(Complex a, Complex b)
	{
		Complex c = new Complex();
		c.r = a.r * b.r - a.i * b.i;
		c.i = a.i * b.r + a.r * b.i;
		return c;
	}

	/****************/
	/* 複素数の除算 */
	/****************/
	public static Complex dev(Complex a, Complex b)
	{
		Complex c = new Complex();
		double x  = 1.0 / (b.r * b.r + b.i * b.i);
		c.r = (a.r * b.r + a.i * b.i) * x;
		c.i = (a.i * b.r - a.r * b.i) * x;
		return c;
	}
}

/*
------------------------data1----------------------------
ゲイン gain 位相 phase
周波数の下限と上限 0.01 100 分割数 100
分子の表現方法 0 次数 0 係数（次数が低い順） 1.0
分母の表現方法 0 次数 3 係数（次数が低い順） 1.0 11.1 11.1 1.0

------------------------data2----------------------------
ゲイン gain 位相 phase
周波数の下限と上限 0.01 100 分割数 100
分子の表現方法 1 次数 0 係数（次数が低い順） 1.0
分母の表現方法 1 次数 3 係数（次数が低い順） 1.0 0.1 1.0 1.0 1.0 10.0
*/
