/****************************/
/* 乱数の発生               */
/*      coded by Y.Suganuma */
/****************************/
import java.io.*;
import java.util.*;

public class Test {

	static Random rn;

	public static void main(String args[]) throws IOException
	{
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

		System.out.print("乱数の初期値を入力してください ");
		int seed = Integer.parseInt(in.readLine());

		rn = new Random(seed);   // 乱数の初期設定
//		rn = new Random();   // 初期値もランダムにしたい場合
/*
          [a b] 区間の一様乱数
*/
		double mean = 0.0;
		double sd   = 0.0;
		double a    = 2.0;
		double b    = 4.0;
		int n       = 10000;

		for (int i1 = 0; i1 < n; i1++) {
			double r = a + (b - a) * unifm_d();
			mean    += r;
			sd      += r * r;
		}

		mean = mean / n;
		sd   = Math.sqrt(Math.abs(n /(n - 1.0) * (sd / n - mean * mean)));

		System.out.printf("一様分布\n");
		System.out.printf("   平均 %f 標準偏差 %f\n", mean, sd);
/*
          平均 m の指数分布
*/
		mean     = 0.0;
		sd       = 0.0;
		double m = 0.5;

		for (int i1 = 0; i1 < n; i1++) {
			double r = exp_d(m);
			mean    += r;
			sd      += r * r;
		}

		mean = mean / n;
		sd   = Math.sqrt(Math.abs(n /(n - 1.0) * (sd / n - mean * mean)));

		System.out.printf("指数分布\n");
		System.out.printf("   平均 %f 標準偏差 %f\n", mean, sd);
/*
          標準正規分布
*/
		mean = 0.0;
		sd   = 0.0;

		for (int i1 = 0; i1 < n; i1++) {
			double r = norm_d(0.0, 1.0);
			mean    += r;
			sd      += r * r;
		}

		mean = mean / n;
		sd   = Math.sqrt(Math.abs(n /(n - 1.0) * (sd / n - mean * mean)));

		System.out.printf("標準正規分布\n");
		System.out.printf("   平均 %f 標準偏差 %f\n", mean, sd);
	}

	/******************************/
	/* [0, 1]区間の一様乱数の発生 */
	/*      rerutn : 乱数         */
	/******************************/
	static double unifm_d()
	{
	   return rn.nextDouble();
	}

	/*****************************/
	/* 平均値 m の指数乱数の発生 */
	/*      m : 平均             */
	/*****************************/
	static double exp_d(double m)
	{
		return -m * Math.log(unifm_d());
	}

	/***********************************/
	/* 正規分布変量の発生              */
	/*      m : 平均                   */
	/*      s : 標準偏差               */
	/*           return : 正規分布変量 */
	/***********************************/
	static double norm_d(double m, double s)
	{
		double x = 0.0;

		for (int i1 = 0; i1 < 12; i1++)
			x += unifm_d();

		x = s * (x - 6.0) + m;

		return x;
	}
}
