/****************************/
/* ファジイ推論             */
/*      coded by Y.Suganuma */
/****************************/
using System;
using System.IO;

class Program
{
	/****************/
	/* main program */
	/****************/
	static void Main(String[] args)
	{
					// エラー
		if (args.Length != 3) {
			Console.WriteLine("***error   入力データファイル名を指定して下さい");
			Environment.Exit(1);
		}

		else {
					// オブジェクトの生成
			Fuzzy fz = new Fuzzy (args[0]);
					// 推論データの読み込み
			char[] charSep = new char[] {' '};
			String[] lines = File.ReadAllLines(args[1]);
			String[] str   = lines[0].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
			int n_val      = int.Parse(str[1]);
			int n_d        = int.Parse(str[3]);
			double[] x     = new double [n_val];
			double[][] xx  = new double [n_val][];
			for (int i1 = 0; i1 < n_val; i1++) {
				xx[i1] = new double [n_d];
				str    = lines[i1+1].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
				for (int i2 = 0; i2 < n_d; i2++)
					xx[i1][i2] = double.Parse(str[i2+1]);
			}
					// 推論とその結果の出力
			StreamWriter OUT = new StreamWriter(args[2]); 
			for (int i1 = 0; i1 < n_d; i1++) {
				for (int i2 = 0; i2 < n_val; i2++) {
					x[i2] = xx[i2][i1];
					OUT.Write(x[i2] + " ");
				}
				double y = fz.Inf(x);   // 推論
				OUT.WriteLine(y);
			}
			OUT.Close();
		}
	}
}

class Fuzzy {

	double[][][] left;   // [i][j][0] : 中心
                         //       [1] : 幅
	double[][] right;   // [i][0] : 中心
                        //    [1] : 幅
	double[] omega;   // ωの値
	int bun;   // 積分の分割数
	int method;   // 推論方法
                  //    =0 : and,or
                  //    =1 : *,or
                  //    =2 : *,+
	int n_rule;   // 規則の数
	int n_val;   // 変数の数
	int[][] rule;   // [i][j] =0 : ｉ番目の規則はｊ番目の変数を使用しない
                    //        =1 : ｉ番目の規則はｊ番目の変数を使用する

	/************************************/
	/* コンストラクタ                   */
	/*      name : 入力データファイル名 */
	/************************************/
	public Fuzzy (String name)
	{
		char[] charSep = new char[] {' '};
		String[] lines = File.ReadAllLines(name);
				//
				// 基本データの入力
				//
		String[] str = lines[0].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
		method       = int.Parse(str[1]);
		n_rule       = int.Parse(str[3]);
		n_val        = int.Parse(str[5]);
		bun          = int.Parse(str[7]);
				//
				// 領域の確保
				//
		omega = new double [n_rule];
		right = new double [n_rule][];
		rule  = new int [n_rule][];
		left  = new double [n_rule][][];
		for (int i1 = 0; i1 < n_rule; i1++) {
			right[i1] = new double [2];
			rule[i1]  = new int [n_val];
			left[i1]  = new double [n_val][];
			for (int i2 = 0; i2 < n_val; i2++)
				left[i1][i2]  = new double [2];
		}
				//
				// 規則データの入力
				//
		for (int i1 = 0; i1 < n_rule; i1++) {
						// 左辺
			for (int i2 = 0; i2 < n_val; i2++) {
				str = lines[(n_val+2)*i1+i2+2].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
				rule[i1][i2] = int.Parse(str[1]);
				if (rule[i1][i2] > 0) {
					left[i1][i2][0] = double.Parse(str[3]);
					left[i1][i2][1] = double.Parse(str[4]);
				}
			}
						// 右辺
			str = lines[(n_val+2)*i1+n_val+2].Split(charSep, StringSplitOptions.RemoveEmptyEntries);
			right[i1][0] = double.Parse(str[1]);
			right[i1][1] = double.Parse(str[2]);
		}
	}

	/*******************************************/
	/* 交点の計算                              */
	/*      x : x                              */
	/*      c : 中心                           */
	/*      h : 幅                             */
	/*      r : <0 : 左辺のメンバーシップ関数  */
	/*          >=0 : 右辺(ω)                 */
	/*      return : 交点                      */
	/*******************************************/
	double Cross(double x, double c, double h, double r)
	{
		double x1, x2, y1, y2;

		if (x < c) {
			x1 = c - h;
			x2 = c;
			y1 = 0.0;
			y2 = (r < 0.0 || (r >= 0.0 && method == 0)) ? 1.0 : r;
		}
		else {
			x1 = c;
			x2 = c + h;
			y1 = (r < 0.0 || (r >= 0.0 && method == 0)) ? 1.0 : r;
			y2 = 0.0;
		}

		double y = y1 + (y2 - y1) * (x - x1) / (x2 - x1);

		if (y < 0.0)
			y = 0.0;
		else {
			if (r >= 0.0 && method == 0 && y > r)
				y = r;
		}

		return y;
	}

	/**********************************************/
	/* 右辺の計算                                 */
	/*      x : 値                                */
	/*      om[i] : <0 : ｉ番目の規則を使用しない */
	/*              >=0 : ωの値                  */
	/*      return : ｘにおける右辺の値           */
	/**********************************************/
	double Height(double x, double [] om)
	{
		double y = 0.0;
		int sw = 0;

		for (int i1 = 0; i1 < n_rule; i1++) {

			if (om[i1] >= 0.0) {

				double y1 = Cross(x, right[i1][0], right[i1][1], om[i1]);

				if (sw == 0) {
					y  = y1;
					sw = 1;
				}

				else {
					if (method < 2) {
						if (y1 > y)
							y = y1;
					}
					else
						y += y1;
				}
			}
		}

		return y;
	}

	/**************************/
	/* 推論の実行             */
	/*      x[i] : 各変数の値 */
	/*      return : 推論値   */
	/**************************/
	public double Inf(double[] x)
	{
				//
				// ωの計算
				//
		for (int i1 = 0; i1 < n_rule; i1++) {

			omega[i1] = -1.0;

			for (int i2 = 0; i2 < n_val; i2++) {

				if (rule[i1][i2] > 0) {

					double x1 = 0.0;
					if (x[i2] > left[i1][i2][0]-left[i1][i2][1] &&
                        x[i2] < left[i1][i2][0]+left[i1][i2][1])
						x1 = Cross(x[i2], left[i1][i2][0], left[i1][i2][1], -1.0);
					else
						x1 = 0.0;

					if (omega[i1] < 0.0)
						omega[i1] = x1;
					else {
						if (method == 0) {
							if (x1 < omega[i1])
								omega[i1] = x1;
						}
						else
							omega[i1] *= x1;
					}
				}
			}
		}
				//
				// 右辺の計算
				//
		double y = Result(omega);

		return y;
	}

	/**********************************************/
	/* 右辺による推論                             */
	/*      om[i] : <0 : ｉ番目の規則を使用しない */
	/*              >=0 : ωの値                  */
	/*      return : 推論値                       */
	/**********************************************/
	double Result(double [] om)
	{
				//
				// 積分範囲と積分幅の計算
				//
		double max = 0.0, min = 0.0, x1, x2, y = 0.0, y1, y2, z1, z2;
		int sw = 0;

		for (int i1 = 0; i1 < n_rule; i1++) {
			if (om[i1] >= 0.0) {
				x1 = right[i1][0] - right[i1][1];
				x2 = right[i1][0] + right[i1][1];
				if (sw == 0) {
					min = x1;
					max = x2;
					sw  = 1;
				}
				else {
					if (x1 < min)
						min = x1;
					if (x2 > max)
						max = x2;
				}
			}
		}

		double h = (max - min) / bun;

		if (h < 1.0e-15) {
			Console.WriteLine("***error  invalid data (h = 0) ***");
			Environment.Exit(1);
		}
				//
				// 積分(重心の計算，台形則)
				//
		z1 = Height(min, om);
		z2 = Height(max, om);
		y1 = 0.5 * (min * z1 + max * z2);
		y2 = 0.5 * (z1 + z2);
		x1 = min;

		for (int i1 = 0; i1 < bun-1; i1++) {
			x1 += h;
			z1  = Height(x1, om);
			y1 += x1 * z1;
			y2 += z1;
		}

		y1 *= h;
		y2 *= h;

		if (Math.Abs(y2) > 1.0e-10)
			y = y1 / y2;
		else {
			Console.WriteLine("***error  invalid data (y2 = 0) ***");
			Environment.Exit(1);
		}

		return y;
	}
}

/*
------------------------規則ファイル--------------
推論方法 0 規則の数 3 変数の数 1 積分の分割数 100
規則１
　変数１ 1 中心と幅 1.0 1.0
　右辺（中心と幅） 1.0 15.0
規則２
　変数１ 1 中心と幅 2.0 1.0
　右辺（中心と幅） 11.0 15.0
規則３
　変数１ 1 中心と幅 3.0 1.0
　右辺（中心と幅） 21.0 15.0

------------------------推論ファイル--------------
変数の数 1 データ数 21
変数１ 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0
*/
